;;; -*- Syntax: Common-lisp; Package: USER -*-

;;;; Testing fact-closing code

;; (from PAE kitchen domain model)

(deffact (set Mobile-Objects))
(deffact (set Places))

(defentity Mobile
  (Mobile-Objects Has-Member ?self) ;; Add to set of mobile objects
  (choice-set (at ?self (:quote ?place)))) ;; It's always somewhere

(defpredicate Place
  (Places Has-Member ?self) ;; Add to set of places
  (iff (clear ?self)
       (every ?m mobile-objects (not (at ?m ?self)))))

(adb::srule :intern ;; A mobile object could be at any place.
      ((Mobile ?m) :var ?f1
       (Place ?p) :var ?f2)
      (adb::refer-to (At ?m ?p))
      (adb::refer-to (not (At ?m ?p))))

(adb::srule :intern ;; Can't be in two places at once
	    ((at ?m ?p1) :var ?f1
	     (at ?m ?p2) :var ?f2 :test (not (equal ?p1 ?p2)))
	    (format t "~% ~A can't be at ~A and ~A." ?m ?p1 ?p2)
	    (adb::rnogood (?f1 ?f2) :Place-Exclusion))

(adb::srule :intern ;; Simplify for now -- one way heat paths
      ((Thermal-Contact ?obj1 ?obj2) :var ?f1)
      (adb:rjustify (Heat-Path ?obj1 ?obj2) (?f1) :Heat-path-Existence)
      (adb:rjustify (Heat-Path ?obj2 ?obj1) (?f1) :Heat-path-Existence)
      )

(defClosed-Predicate Thermal-Contact)
(defClosed-Predicate Heat-Path) ;; Pick up non-existence by failure

(defComputable-Relation Heat-Aligned)
(defClosed-Predicate Heat-Aligned)

(adb::srule :intern ;; Heat path alignment
      ((Heat-Path ?obj1 ?obj2) :var ?f1)
      (adb:rjustify (Heat-Aligned (Heat-Path ?obj1 ?obj2)) (?f1)
		    :Heat-Alignment-Definition))

;;;; Paths and alignment
(adb::srule  :intern ;; Heat path existence (assumes no thermal insulators)
      ((at ?m ?p) :var ?f1
       (Quantity (Heat ?m)) :var ?f2
       (Quantity (Heat ?p)) :var ?f3)
      (format t "~%Justifying thermal contact from ~A to ~A."
	      (adb::lprinc-string ?m)
	      (adb::lprinc-string ?p))
      (adb:rjustify (Thermal-Contact ?m ?p)
		    (?f1 ?f2 ?f3) :Thermal-Contact-Definition))

;; The dynamics

(defquantity-type heat individual)
(defquantity-type temperature individual)
(defquantity-type flow-rate individual)

(defentity thermal-object
  ;; Continuous properties shared by all physobs
  (quantity (heat ?self))
  (quantity (temperature ?self))
  (Qprop (temperature ?self) (heat ?self))
  (Greater-than (A (temperature ?self)) ZERO))

(defprocess (Heat-Flow ?src ?dst ?path)
	    Individuals ((?src :conditions (Quantity (Heat ?src)))
			 (?dst :conditions (Quantity (Heat ?dst))
			       (Heat-Path ?src ?dst))
			 (?path :bind (Heat-Path ?src ?dst)))
	    Preconditions ((heat-aligned ?path))
	    QuantityConditions ((greater-than (A (temperature ?src))
					      (A (temperature ?dst))))
	    Relations ((quantity flow-rate)
		       (Q= flow-rate (- (temperature ?src) 
					(temperature ?dst))))
	    Influences ((I+ (heat ?dst) (A flow-rate))
			(I- (heat ?src) (A flow-rate))))
