;;; -*- Mode: LISP; Syntax: Common-lisp; Package: USER -*-


(in-package qpe::*user-package*)

(adb::rules-file "NST-DOMAIN")

;;;; Test Naive Steam Theory for QPE
;;
;; This version is for testing the Common Lisp implementation.
;;Some geometric properties have been left out.
;; QPE V2 has flushed the LOCAL construct, and is simply more clever about
;; creating local names.
;; Also, individual specs have been made more useful

(defQuantity-Type Amount-of-in Constant Constant Individual)
(defQuantity-Type Amount-of Individual)
(defQuantity-Type Heat Individual)
(defQuantity-Type Temperature Individual)
(defQuantity-Type Pressure Individual)
(defQuantity-Type Volume individual)
(defQuantity-Type Flow-Rate Individual)
(defQuantity-Type Level Individual)
(defQuantity-Type Max-Height Individual)
(defQuantity-Type Tboil Constant Individual)
(defQuantity-Type Absorbtion Individual)
(defQuantity-Type Generation-Rate Individual)
(defQuantity-Type Restorative Individual)
(defQuantity-Type bottom-height individual)
(defQuantity-Type Top-height individual)

(adb:Rule :INTERN (((container ?c) . :TRUE)
		   ((substance ?s) . :TRUE)
		   ((state ?st) . :TRUE))
	  ;; Each container could contain each substance in every state.
	  (adb:rassert! ((Can-Contain-Substance ?c ?s ?st) . :TRUE)) 
	  ;; amount-of-in denotes the (possibly zero) amount that there is. 
	  (adb:rassert! ((quantity (amount-of-in ?s ?st ?c)) . :TRUE))
	  ;; amount-of-in is never negative.
	  (adb:rassert! ((less-than (a (amount-of-in ?s ?st ?c)) zero) . :FALSE))
	  ;; There is a boiling point for that substance in that container
	  ;; (Assumes uniformity, ignores sublimation, etc.)
	  (adb:rassert! ((quantity (Tboil ?s ?c)) . :TRUE))
	  ;; The boiling point also is never negative.
	  (adb:rassert! ((Greater-Than (a (Tboil ?s ?c)) zero) . :TRUE)))

(defentity (Contained-Stuff (C-S ?sub liquid ?can)) 
	   (Contained-Liquid (C-S ?sub liquid ?can)))

(defentity (Contained-Liquid (C-S ?sub liquid ?can))
	   ;;; Liquids have a novel quantity, level.
	   (quantity (level (C-S ?sub liquid ?can)))
	   ;;; Bound temperature from above.
	   (not (Greater-Than (A (temperature (C-S ?sub liquid ?can)))
			      (A (Tboil ?sub ?can))))
	   ;;; Now specify some functional relationships
	   ;;; These have been changed to amount-of-in while I figure out
	   ;;; how to make the intermediate steps work right, should be Amount-of
	   (Function-Spec Level-Function
			  (Qprop (level (C-S ?sub liquid ?can))
				 (Amount-of (C-S ?sub liquid ?can))))
	   (Correspondence ((A (level (C-S ?sub liquid ?can)))
			    (A (bottom-height ?can)))
			   ((A (amount-of (C-S ?sub liquid ?can))) zero)) 
	   ;;; This should depend on the container being open
	   (Function-Spec P-L-Function
			  (Qprop (pressure (C-S ?sub liquid ?can))
				 (level (C-S ?sub liquid ?can)))))

(defentity (Contained-Stuff (C-S ?sub gas ?can))
	   (Contained-Gas (C-S ?sub gas ?can)))

(defentity (Contained-Gas (C-S ?sub gas ?can))
	   ;;; More information for matching
	   (((C-S ?sub gas ?can) substance ?sub) . :TRUE)
	   (((C-S ?sub gas ?can) container ?can) . :TRUE)
	   ;;; A Gas cannot be less than the boiling temperature of the stuff
	   (not (less-than (A (temperature (C-S ?sub gas ?can)))
			   (A (Tboil ?sub ?can))))
	   ;;; The next qprops express the ideal gas law
	   ;;; The first one is diked out until resolution rules are implemented. 
	 ;  (Qprop- (temperature (C-S ?sub gas ?can)) (pressure (C-S ?sub gas ?can)))
	   (Qprop (pressure (C-S ?sub gas ?can)) (amount-of (C-S ?sub gas ?can)))
	   (Qprop- (pressure (C-S ?sub gas ?can)) (volume (C-S ?sub gas ?can)))
	   (Qprop (pressure (C-S ?sub gas ?can)) (heat (C-S ?sub gas ?can))))

(defentity physob
	   ;;; Main characteristic is that it has a number of quantities
  (quantity (Amount-of ?self))
  (quantity (heat ?self))
  (quantity (temperature ?self))
  (quantity (pressure ?self))  
  (quantity (volume ?self))
  ;;; There are a few state-independent relationships
  (Qprop (temperature ?self) (heat ?self))
  (not (less-than (a (Amount-of ?self)) zero)))

(defentity container
	   ;;; Simple container geometry only has heights of bottoms and tops
	   (quantity (bottom-height ?self))
           (quantity (top-height ?self))
	   (greater-than (A (top-height ?self)) (A (bottom-height ?self))))

(defentity Fluid-Path
	   (quantity (max-height ?self)))

(defentity Heat-Path)

(defentity Temperature-Source 
  (quantity (heat ?self))
  (quantity (temperature ?self)))

;;;; View vocabulary

(defview (Contained-Stuff (C-S ?s ?st ?c))
	 Individuals ((?c :type container)
		      (?s :type substance)
		      (?st :type state))
	 Preconditions ((Can-Contain-Substance ?c ?s ?st))
	 QuantityConditions ((greater-than (A (Amount-of-in ?s ?st ?c)) ZERO))
	 Relations ((there-is-unique (C-S ?s ?st ?c))
		    (Q= (amount-of (C-S ?s ?st ?c)) (amount-of-in ?s ?st ?c))
		    (qprop (amount-of (C-S ?s ?st ?c)) (amount-of-in ?s ?st ?c))
		    (physob (C-S ?s ?st ?c))))

;;;; Process vocabulary

(defprocess (Liquid-flow ?sub ?src ?dst ?path)
	    Individuals ((?sub :type Substance)
			 (?src :type Container)
			 (?dst :type Container)
			 (?src-cl :bind (C-S ?sub LIQUID ?src))
			 (?dst-cl :bind (C-S ?sub LIQUID ?dst))
			 (?path :type Fluid-Path
				:conditions
				(Fluid-Connection ?path ?src ?dst)))
	    Preconditions ((aligned ?path))
	    QuantityConditions
	    ((greater-than (A (pressure ?src-cl)) (A (pressure ?dst-cl))))
	    Relations ((quantity flow-rate)
		       (Q= flow-rate (- (pressure ?src-cl) (pressure ?dst-cl)))
		       (filled ?path)
		       (greater-than (A flow-rate) zero))
	    Influences ((I+ (Amount-of-in ?sub LIQUID ?dst) (A flow-rate))
			(I- (Amount-of-in ?sub LIQUID ?src) (A flow-rate))))

(defprocess (Heat-Flow ?src ?dst ?path)
	    Individuals ((?src :conditions (Quantity (Heat ?src)))
			 (?dst :conditions (Quantity (Heat ?dst)))
			 (?path :type Heat-Path
				:conditions
				(Heat-Connection ?path ?src ?dst)))
	    Preconditions ((heat-aligned ?path))
	    QuantityConditions ((greater-than (A (temperature ?src))
					      (A (temperature ?dst))))
	    Relations ((quantity flow-rate)
		       (Q= flow-rate (- (temperature ?src) 
					(temperature ?dst)))
		       (greater-than (A flow-rate) zero))
	    Influences ((I+ (heat ?dst) (A flow-rate))
			(I- (heat ?src) (A flow-rate))))

(defprocess (Boiling ?w ?hf)
	    Individuals ((?w :type Contained-Liquid
			     :form (C-S ?s LIQUID ?c))
			 (?hf :conditions ((Process-Instance Heat-Flow) ?hf)
			      (?hf dst ?w))) 
	    QuantityConditions ((Active ?hf)
				(not (greater-than (A (Tboil ?s ?c))
						   (A (Temperature ?w)))))
	    Relations ((quantity generation-rate)
		       (quantity absorbtion)
		       (when (Exists (C-S ?s gas ?c))
			 (Equal-To (d (heat (c-s ?s gas ?c))) zero)
			 (Q= (temperature (c-s ?s GAS ?c)) (temperature ?w)))
		       (Greater-than (A absorbtion) ZERO)
		       (Greater-than (A generation-rate) ZERO)
		       (Qprop generation-rate (flow-rate ?hf)))
	    Influences ((I- (heat ?w) (A (flow-rate ?hf)))
			(I- (heat (c-s ?s gas ?c)) (A absorbtion))
			(I+ (Amount-of-in ?s GAS ?c) (A generation-rate))
			(I- (Amount-of-in ?s LIQUID ?c) (A generation-rate))))

(defprocess (heat-replenish ?source)
	    Individuals ((?source :type Heat-Source
				  :conditions (quantity (heat ?source))))
	    Preconditions ((Heat-Source ?source))
	    QuantityConditions ()
	    Relations ((quantity Restorative) 
		       (Equal-to (D (Heat ?source)) ZERO))
	    Influences ((I+ (heat ?source) (A restorative))))

