;;; -*- Syntax: Common-lisp; Mode: LISP; -*-

(in-package *user-package*)

;;;; Simple "domain model" for spring-block oscillator

(defQuantity-Type Position individual)
(defQuantity-Type Velocity individual)
(defQuantity-type Force individual)
(defQuantity-Type Length individual)
(defQuantity-Type Rest-Length individual)
(defQuantity-Type acc individual)

;;;; Objects and connectivity

(defentity Physob
  (quantity (position ?self))
  (quantity (velocity ?self))
  (quantity (force ?self)))

(defentity Block
  (Physob ?self)
  (Mobile ?self))

(defpredicate (Connected-to-spring ?from ?to)
  (Q= (force ?from) (force ?to))
  (Qprop (length ?to) (position ?from))
  (Correspondence ((A (length ?to)) (A (rest-length ?to)))
		  ((A (position ?from)) zero)))

;;;; Spring model

(defentity spring
  (quantity (force ?self))
  (quantity (rest-length ?self))
  (quantity (length ?self))
  (qprop- (force ?self) (length ?self))
  (correspondence ((A (force ?self)) zero)
		  ((A (length ?self)) (A (rest-Length ?self)))))

(defview (Relaxed ?s)
  Individuals ((?s :type spring))
  QuantityConditions ((Equal-to (A (length ?s)) (A (rest-length ?s)))))

(defview (Stretched ?s)
  Individuals ((?s :type spring))
  QuantityConditions ((Greater-than (A (length ?s)) (A (rest-length ?s)))))

(defview (Compressed ?s)
  Individuals ((?s :type spring))
  QuantityConditions ((Less-than (A (length ?s)) (A (rest-length ?s)))))

;;;; Process vocabulary

(defProcess (Motion ?obj)
  Individuals ((?obj :type physob
		     :conditions (mobile ?obj)))
  QuantityConditions ((not (Equal-to (A (velocity ?obj)) zero)))
  Influences ((I+ (Position ?obj) (A (velocity ?obj)))))

(defProcess (Accelerating ?obj)
   Individuals ((?obj :type physob
		      :conditions (mobile ?obj)))
   QuantityConditions ((not (Equal-to (A (force ?obj)) zero)))
   Relations ((Quantity acc)
	      (Q= acc (force ?obj)))
   Influences ((I+ (velocity ?obj) (A acc))))
